'/* VB.net Port of DotNetHelpServices
' * =============
' * 
' * DotNetHelpServices (http://jonathanmalek.com/)
' * Copyright  2004 Jonathan Malek. All Rights Reserved.
' * 
' * Permission is hereby granted, free of charge, to any person obtaining 
' * a copy of this software and associated documentation files (the "Software"), 
' * to deal in the Software without restriction, including without limitation 
' * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
' * and/or sell copies of the Software, and to permit persons to whom the 
' * Software is furnished to do so, subject to the following conditions:
' * 
' * The above copyright notice and this permission notice shall be included in all
' * copies or substantial portions of the Software.
' * 
' * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
' * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
' * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
' * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
' * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
' * THE SOFTWARE.
'*/

Imports System.Xml
Imports System.Runtime.InteropServices
Imports HelpServiceTypeLib
Imports System.Text.RegularExpressions

Public Class HelpServicesClasses
    ' Methods
    Public Sub New()

    End Sub
    Public Function FormatIncident(ByVal ticket As String, ByVal length As Integer) As XmlDocument
        Dim incident1 As New Incident
        incident1.Ticket = ticket
        incident1.Length = length
        incident1.Unsolicited = True
        Dim document1 As New System.Xml.XmlDocument
        document1.LoadXml(incident1.ToString)
        Return document1
    End Function

    Public Function GenerateTicket(ByVal MachineName As String, ByVal UserName As String, ByVal DomainName As String, ByVal SessionID As Integer, ByVal UserHelpBlob As String) As String
        Dim type1 As Type = Type.GetTypeFromProgID("PCH.HelpService", MachineName, True)
        Dim obj1 As Object = Activator.CreateInstance(type1)
        Dim service1 As IPCHService = CType(obj1, IPCHService)
        Dim text1 As String = service1.RemoteConnectionParms(UserName, DomainName, SessionID, UserHelpBlob)
        Marshal.ReleaseComObject(obj1)
        Return text1
    End Function

    Public Function GetSessions(ByVal MachineName As String) As Session()
        Dim type1 As Type = Type.GetTypeFromProgID("PCH.HelpService", MachineName, True)
        Dim obj1 As Object = Activator.CreateInstance(type1)
        Dim service1 As IPCHService = CType(obj1, IPCHService)
        Dim collection1 As IPCHCollection = Nothing
        Try
            collection1 = service1.RemoteUserSessionInfo
        Catch exception1 As COMException
            Throw New ApplicationException("Could not execute call on remote system.  Most likely the user of the remote system has 'Unsolicited' turned off.", exception1)
        End Try
        If (collection1 Is Nothing) Then
            Throw New ApplicationException("The user you are trying to help has 'Unsolicited' turned off.")
        End If
        If (collection1.Count = 0) Then
            Throw New ApplicationException("There are no users logged in to the machine.")
        End If
        Dim enumerator1 As IEnumerator = collection1.GetEnumerator
        Dim list1 As New ArrayList
        Dim num1 As Integer = 0
        Do While (num1 < collection1.Count)
            enumerator1.MoveNext()
            Dim session1 As ISAFSession = TryCast(enumerator1.Current, ISAFSession)
            Dim session2 As New Session
            session2.UserName = session1.UserName
            session2.DomainName = session1.DomainName
            session2.SessionID = CType(session1.SessionID, Integer)
            session2.SessionState = session1.SessionState.ToString
            list1.Add(session2)
            num1 += 1
        Loop
        Marshal.ReleaseComObject(obj1)
        Return CType(list1.ToArray(GetType(Session)), Session())
    End Function

End Class

Public Class Incident
    ' Methods
    Public Sub New()
        Me.Unsolicited = True
        Me.Length = 60
        Me.StartTime = DateTime.Now
        Me.LocalNetwork = False
    End Sub

    Public Function GetUnixTime(ByVal dateTime As DateTime) As Integer
        Dim time2 As New DateTime(1970, 1, 1, 0, 0, 0, 0)
        Dim time1 As DateTime = time2.ToLocalTime
        Dim span1 As TimeSpan = dateTime.Subtract(time1)
        Return CType(span1.TotalSeconds, Integer)
    End Function

    Public Overrides Function ToString() As String
        Dim writer1 As New IO.StringWriter
        Dim writer2 As New XmlTextWriter(writer1)
        writer2.Formatting = Formatting.Indented
        writer2.Indentation = 1
        writer2.IndentChar = Char.Parse(ChrW(9))
        writer2.WriteStartDocument()
        writer2.WriteStartElement("UPLOADINFO")
        writer2.WriteAttributeString("", "TYPE", "", "Escalated")
        writer2.WriteStartElement("UPLOADDATA")
        writer2.WriteAttributeString("", "RCTICKET", "", Me.Ticket)
        writer2.WriteAttributeString("", "RCTICKETENCRYPTED", "", "0")
        Dim num1 As Integer = Me.GetUnixTime(Me.StartTime)
        writer2.WriteAttributeString("", "DtStart", "", num1.ToString)
        writer2.WriteAttributeString("", "DtLength", "", Me.Length.ToString)
        If Me.Unsolicited Then
            writer2.WriteAttributeString("", "URA", "", "1")
        End If
        writer2.WriteEndElement()
        writer2.WriteEndElement()
        writer2.WriteEndDocument()
        Return writer1.GetStringBuilder.ToString
    End Function

    ' Properties
    Public Property Length() As Integer
        Get
            Return Me._length
        End Get
        Set(ByVal value As Integer)
            Me._length = value
        End Set
    End Property

    Public Property LocalNetwork() As Boolean
        Get
            Return Me._localNetwork
        End Get
        Set(ByVal value As Boolean)
            Me._localNetwork = False
        End Set
    End Property

    Public Property StartTime() As DateTime
        Get
            Return Me._startTime
        End Get
        Set(ByVal value As DateTime)
            Me._startTime = value
        End Set
    End Property

    Public Property Ticket() As String
        Get
            Dim services1 As New IpServices
            If (services1.BehindNat And Not Me.LocalNetwork) Then
                Dim textArray1 As String() = Me._ticket.Split(New Char() {","c})
                Dim text2 As String = textArray1(2)
                Dim text3 As String = text2.Split(New Char() {";"c})(0).Split(New Char() {":"c})(1)
                textArray1(2) = (services1.IPAddress.ToString & ":" & text3)
                Return String.Join(",", textArray1)
            End If
            Return Me._ticket
        End Get
        Set(ByVal value As String)
            Me._ticket = value
        End Set
    End Property

    Public Property Unsolicited() As Boolean
        Get
            Return Me._unsolicited
        End Get
        Set(ByVal value As Boolean)
            Me._unsolicited = value
        End Set
    End Property

    Public Property UserName() As String
        Get
            Return Me._userName
        End Get
        Set(ByVal value As String)
            Me._userName = value
        End Set
    End Property

    ' Fields
    Private _length As Integer
    Private _localNetwork As Boolean
    Private _startTime As DateTime
    Private _ticket As String
    Private _unsolicited As Boolean
    Private _userName As String
    Private Const RCTICKET_ADDRESSES As Integer = 2
End Class

Public Class IpServices
    ' Methods
    Public Sub New()
        Dim text1 As String = Net.Dns.GetHostName
        Me._hostEntry = Net.Dns.GetHostEntry(text1)
        Me.ConnectToGetIPAddress()
        Me._behindNat = True
        Dim addressArray1 As Net.IPAddress() = Me._hostEntry.AddressList
        Dim num1 As Integer = 0
        Do While (num1 < addressArray1.Length)
            Dim address1 As Net.IPAddress = addressArray1(num1)
            If address1.Equals(Me._usedIPAddress) Then
                Me._behindNat = False
                Return
            End If
            num1 += 1
        Loop
    End Sub

    Private Sub ConnectToGetIPAddress()
        If S_CorpNet = True Then
            'We're on a managed network, ignore potential NATs.
            Dim ipHostEntry As Net.IPHostEntry = Net.Dns.GetHostEntry(Net.Dns.GetHostName())
            Dim ipAddr As Net.IPAddress() = ipHostEntry.AddressList
            Debug.Print(ipAddr(0).ToString)
            Me._usedIPAddress = ipAddr(0)
        Else
            'We're not on a managed network.
            Try
                '//Note: if the whatismyip.com web service changes, the regex value may need updating.
                Dim client1 As New Net.WebClient
                Dim buffer1 As Byte() = client1.DownloadData("http://www.whatismyip.com/automation/n09230945.asp")
                Dim text1 As String = System.Text.Encoding.ASCII.GetString(buffer1)
                Me._usedIPAddress = Net.IPAddress.Parse(text1)
                '//Debug.Print(Me._usedIPAddress.ToString)
            Catch fmt_ex As System.FormatException
                Dim msg As String
                msg = "This version of Remote Assistance Helper has become obsolete or an error occured" & vbCrLf
                msg &= "while determining this computer's IP address.  Please call for support."
                msg &= vbCrLf & vbCrLf & "The application will now exit."
                MessageBox.Show(msg, "A fatal error has occured.", MessageBoxButtons.OK, MessageBoxIcon.Error)
                Application.ExitThread()
                Application.Exit()

            Catch ex As Exception
                Dim msg As String
                msg = "A fatal error occured while Remote Assistance Helper was determining your IP address" & vbCrLf
                msg &= "Please call for support."
                msg &= vbCrLf & vbCrLf & "The application will now exit."
                MessageBox.Show(msg, "A fatal error has occured.", MessageBoxButtons.OK, MessageBoxIcon.Error)
                Application.ExitThread()
                Application.Exit()

            End Try
        End If
    End Sub

    ' Properties
    Public ReadOnly Property BehindNat() As Boolean
        Get
            Return Me._behindNat
        End Get
    End Property

    Public ReadOnly Property HostName() As String
        Get
            Return Me._hostEntry.HostName
        End Get
    End Property

    Public ReadOnly Property IPAddress() As Net.IPAddress
        Get
            Return Me._usedIPAddress
        End Get
    End Property

    ' Fields
    Private _behindNat As Boolean
    Private _hostEntry As Net.IPHostEntry
    Private _usedIPAddress As Net.IPAddress
End Class

Public Class Session
    ' Methods
    Public Sub New()

    End Sub

    ' Fields
    Public DomainName As String
    Public SessionID As Integer
    Public SessionState As String
    Public UserName As String
End Class



